#!/usr/bin/perl

package GenebankParser;

#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Parser for genebank files
# NOTE: genebank gives end position of genes as last index of stopcodon
# start is given as first base of the startcodon  
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
use Bio::SeqIO;
use Bio::Seq;
use YACOP::gene_pred::DataHandle;

my $TYPE = "GenebankParser"; 
my $NAME = "GeneBank";
my $DIFF = 2; #difference genebank output, StopCodon

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Whole data from given file is read and stored as Element-objects to a new
# created DataHandle. Start is the first base after start codon, end 
# position is last index of terminator. end is changed by $DIFF nucleotides
# to first index of terminator.
# score -
# comment is annotation
#@param filename of genebank data
#@param $shift (if genome data has been shifted. $shift will be substracted
#              from found position in GenBank-file.) 
#@return current DataHandle 
#------------------------
sub parse{
    my ($self, $file) = @_;
    my $data_handle = DataHandle->new($TYPE);
    my (@array, @tmp, $str, @rbs);
    $counter =1;
    open(g_input, $file) 
	|| die "cannot open genebank input file ".$file."\n";
    while (<g_input>){last if($_ =~ /FEATURES/);}
    while (<g_input>){
	@array = split;
	last if($array[0] =~ /ORIGIN/);
	if ($array[0] =~ /\bCDS\b/ && $array[1] =~/../){
	    undef @tmp;
	    if ($array[1] =~ /complement/){        #start-stop
		$array[1] =~ /\d+/;
		$tmp[1]=$&;
		$array[1]=~ /\d+\)+/;     
		$& =~ /\d+/;
		$tmp[0]=$&;
	    }
	    else {
		$array[1] =~ /\d+/;                  
		$tmp[0]=$&;                        #fwd
		$array[1] =~ /\d+$/;
		$tmp[1] =$&;
		if ($array[1] =~ /join/){
		    $array[1]=~ /\d+\)+/;     
		    $& =~ /\d+/;
		    $tmp[1]=$&;
		}
	    }
	    while (<g_input>){
		if(/\/translation/ || /\/db_xref/){last;}
		elsif(/\/gene/){             #id
		    /\d+/;
		    push @tmp, $&;}
		elsif (/\/product/){         #anno
		    /[\"]/;
		    my $str = $';               #postmatch
		    chomp $str;
		    if ($str =~ /\"/){chop$str;}
		    else {
			my $s = <g_input>;
			$s =~ /\S+/;
			$str.=" ";
			$str.=$&;
			chomp $str;
			chop $str;}
		    push @tmp, $str;
		}
	    }
	    my $com = $self->complement($tmp[0], $tmp[1]);
	    if ($com=~/F/i){$diff=-$DIFF}
	    else{$diff=$DIFF}
	    $data_handle->add_element($counter++, $tmp[0], ($tmp[1]+$diff), $com, 
				      $tmp[2], $tmp[3]);
	    if ($rbs){
		$data_handle->get_element($counter-1)->rbs($rbs);
		undef $rbs;
	    }
	}
        if ($array[0]=~/\bRBS\b/){                 #RBS
	    $array[1]=~s/complement\(//i;
	    $array[1]=~s/\)//i;
	    $rbs=$array[1];
	}
    }
    close(g_input);
    return $data_handle;
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Calulates the complement of given start - end position. 
# (for deprecated routin with orf nr see CriticaParser::complement_no())
# @param $start
# @param $end
# @return complement (format: F2)
#---------------------------
sub complement{
    my ($self, $start, $end) = @_;
    return 'F' if ($start<$end);
    return 'R';
}

sub name{
    return $NAME;
}

sub type{
    return $TYPE;
}
return 1;

