#!/usr/bin/perl

package GlimmerNudge;
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# GlimmerNudge - Starter class for glimmer2.02 or glimmer2.10
# (GlimmerParser.pm has to be modified.) 
# offers routines to start glimmer sub processes automated or manually.
# NOTE: Processes are not startet as batch processes. It's not
#       necessary caus runtime takes only minutes for each subprocess.
#script gl2rbs is needed for conversion of glimmer to RBSfinder-format
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

@ISA=(Nudge);
$|=1;
$C_SHIFT=3; #shift of critica stop to long orfs format
  
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @param FASTA
# @param OUTPUT_DIR
# @param PREFIX
#---------------------
sub new{
    my $self = shift;
    $self = $self->SUPER::new(shift, shift, shift);
    $self->{RESULT_OUTPUT} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
                             .".rbsout";
    $self->{GLIMMER_OUT} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
                            .".out";
    $self->{LO_OUT} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}.".orfs";
    $self->{EXTRACT}= $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
                      .".extract";
    $self->{ICM} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
                   .".model";
    #$self->{MSG_LO} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
    #                .".msg_lo";
   #$self->{MSG_EXTR} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
   #                   .".msg_extr";
   # $self->{MSG_BICM} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
   #                   .".msg_bicm";
    $self->{MSG_GL} = $self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}
                      .".msg_gl";
    return $self;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts subprocess long-orfs
#---------------------
sub start_longorfs{
    my $self = shift;
   system $self->{PARAMETER}{glimmer_home}."/long-orfs ".$self->{FASTA}
          .$self->{PARAMETER}{glimmer_l}.$self->{PARAMETER}{longorf_g}
          .$self->{PARAMETER}{longorf_o}.$self->{PARAMETER}{longorf_p}
          ." > ".$self->{LO_OUT}." 2>>".$self->{MSG_GL}

    #." 2> ".$self->{MSG_LO};
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# converting given critica output file to glimmer trainingsset.
# original idea of Alice C. McHardy (Development of joint application 
# strategies for two microbial gene finders, Bioinformatics, Vol. 20 no. 10 
# 2004, pages 1622-1631) 
#---------------------
sub set_longorfs{
    my $self=shift;
    my $c_file=shift;
    my $minlen= 0;
    my $pval=0;
    my $i=1;

    $c_file=~s/-c\s*//;
    open crit, $c_file || die "cannot open critica output file: "
	.$c_file."\n";
    open lo_out, ">".$self->{LO_OUT} || die "cannot write long_orfs ".
	"output file: ".$self->{LO_OUT}."\n"; 
    print lo_out "Putative genes:\n";

    $minlen=$self->{PARAMETER}{gr_crit_minlen} 
            if($self->{PARAMETER}{gr_crit_minlen});
    $pval=$self->{PARAMETER}{gr_crit_maxpval}
          if($self->{PARAMETER}{gr_crit_maxpval});
    while(<crit>){     ## convert crit -> glimmer orfs
	my @t = split;
	$t[3]=~s/.*e//;
	if($t[1]<$t[2]){$stop=$t[2]-$C_SHIFT} 
	else{$stop=$t[2]+$C_SHIFT}
	print lo_out "C".$i."  ".$t[1]."  ".$stop."\n"
	    if(abs($t[1]-$stop)>=$minlen && $t[3]<=$pval);
	$i++;
    }
    close lo_out;
    close crit;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts subprocess extract
#-----------------------
sub start_extract{
    my $self = shift;
    system $self->{PARAMETER}{glimmer_home}."/extract ".$self->{FASTA}." "
	  .$self->{LO_OUT}.$self->{PARAMETER}{extract_skip}
          .$self->{PARAMETER}{extract_l}
          ." > ".$self->{EXTRACT}." 2>>".$self->{MSG_GL};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts subprocess build-icm
#-----------------------
sub start_buildicm{
    my $self = shift;
    system $self->{PARAMETER}{glimmer_home}."/build-icm < ".$self->{EXTRACT}
    ." > ".$self->{ICM}." 2>>".$self->{MSG_GL};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts subprocess glimmer2
#-----------------------
sub start_glimmer{
    my $self = shift;
    system $self->{PARAMETER}{glimmer_home}."/glimmer2 ".$self->{FASTA}." "
	.$self->{ICM}.$self->{PARAMETER}{glimmer_l}
        .$self->{PARAMETER}{glimmer_f}
        .$self->{PARAMETER}{glimmerf}.$self->{PARAMETER}{glimmer_p}
        .$self->{PARAMETER}{glimmer_o}.$self->{PARAMETER}{glimmer_g}
        .$self->{PARAMETER}{glimmer_r}.$self->{PARAMETER}{glimmerr}
        .$self->{PARAMETER}{glimmer_s}.$self->{PARAMETER}{glimmer_t}
        .$self->{PARAMETER}{glimmer_i}.$self->{PARAMETER}{glimmer_w}
    ." > ".$self->{GLIMMER_OUT}." 2>>".$self->{MSG_GL};
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# getting/setting glimmer output
#-----------------------
sub glimmer_out{
    my $self = shift;
    my $tmp = shift;
    $self->{GLIMMER_OUT} = $tmp if ($tmp);
    return $self->{GLIMMER_OUT};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# 
#-----------------------
sub start_RBSfinder{
    my $self = shift;
    my $rbs_tmp = $self->{OUTPUT_DIR}."/".$self->{PREFIX}."_tmp.gl2rbs"; 
#    system "perl ".$self->{PARAMETER}{rbs_home}."/gl2rbs "
    system "gl2rbs ".$self->{GLIMMER_OUT}." ".$rbs_tmp;
    system "perl ".$self->{PARAMETER}{rbs_home}."/rbs_finder.pl "
#          .$self->{FASTA}." ".$self->{GLIMMER_OUT}." "
	.$self->{FASTA}." ".$rbs_tmp." "
          .$self->{RESULT_OUTPUT}." "
	  .$self->{PARAMETER}{RBS_window_size}." "
          .$self->{PARAMETER}{RBS_consensus_seq}." &> "
          .$self->{OUTPUT_DIR}."/gl_".$self->{PREFIX}.".rbs_msg";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# evaluation of given properties.
# @see glimmer documentation and ini file
# @param properties
#----------------------
sub set_parameter{
    my $self = shift;
    $self->{PARAMETER} = shift;

    if (!$self->{PARAMETER}{glimmer_home}){
	return 0;}
    if ($self->{PARAMETER}{glimmer_l} eq "true"){     #glimmer_l
	$self->{PARAMETER}{glimmer_l} = " -l ";}
    else { undef $self->{PARAMETER}{glimmer_l};}

    if ( $self->{PARAMETER}{longorf_g}){       #longorf_g
	$self->{PARAMETER}{longorf_g} = " -g ".
                                       $self->{PARAMETER}{longorf_g};}

    if ( $self->{PARAMETER}{longorf_o}){        #longorf_o
	$self->{PARAMETER}{longorf_o} = " -o "
                                       .$self->{PARAMETER}{longorf_o};}

    if ( $self->{PARAMETER}{longorf_p}){       #longorf_p
	$self->{PARAMETER}{longorf_p} = " -p "
                                        .$self->{PARAMETER}{longorf_p};}

    if ( $self->{PARAMETER}{extract_l}){       #extract_l
	$self->{PARAMETER}{extract_l} = " -l "
                                        .$self->{PARAMETER}{extract_l};}

    if ($self->{PARAMETER}{extract_skip} eq "true"){  #extract_skip
	$self->{PARAMETER}{extract_skip} = " -skip ";}
    else { undef $self->{PARAMETER}{extract_skip};}

    if ($self->{PARAMETER}{glimmer_f} eq "true"){     #glimmer_f
	$self->{PARAMETER}{glimmer_f} = " -f ";}
    else { undef $self->{PARAMETER}{glimmer_f};}

    if ($self->{PARAMETER}{glimmerf} eq "true"){     #glimmer+f 
	$self->{PARAMETER}{glimmerf} = " +f ";}
    else { undef $self->{PARAMETER}{glimmerf};}
    
    if ( $self->{PARAMETER}{glimmer_i}){       #glimmer_i
	$self->{PARAMETER}{glimmer_i} = " -i "
                                         .$self->{PARAMETER}{glimmer_i};}
	
    if ( $self->{PARAMETER}{glimmer_o}){       #glimmer_o
	$self->{PARAMETER}{glimmer_o} = " -o "
                                          .$self->{PARAMETER}{glimmer_o};}

    if ( $self->{PARAMETER}{glimmer_p}){       #glimmer_p
	$self->{PARAMETER}{glimmer_p} = " -p "
                                          .$self->{PARAMETER}{glimmer_p};}

    if ( $self->{PARAMETER}{glimmer_g}){       #glimmer_g 
	$self->{PARAMETER}{glimmer_g} = " -g "
                                          .$self->{PARAMETER}{glimmer_g};}
    
    if ($self->{PARAMETER}{glimmer_r} eq "true"){     #glimmer_r
	$self->{PARAMETER}{glimmer_r} = " -r ";}
    else { undef $self->{PARAMETER}{glimmer_r};}

    if ($self->{PARAMETER}{glimmerr} eq "true"){     #glimmer+r 
	$self->{PARAMETER}{glimmerr} = " +r ";}
    else { undef $self->{PARAMETER}{glimmerr};}

    if ( $self->{PARAMETER}{glimmer_s}){       #glimmer_s 
	$self->{PARAMETER}{glimmer_s} = " -s "
                                        .$self->{PARAMETER}{glimmer_s};}

    if ( $self->{PARAMETER}{glimmer_t}){       #glimmer_t 
	$self->{PARAMETER}{glimmer_t} = " -t "
                                        .$self->{PARAMETER}{glimmer_t};}

    if ( $self->{PARAMETER}{glimmer_w}){       #glimmer_w
	$self->{PARAMETER}{glimmer_w} = " -w "
                                        .$self->{PARAMETER}{glimmer_w};}
    return 1;
}
return 1;

