#!/usr/bin/perl

package GlimmerRBSParser;

#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Parser for glimmer RBSfinder output files
# notation of rbsfinder:
#      end position of putative genes is given as last position before stop
#      (->exclusive terminal codon) 
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
use YACOP::gene_pred::DataHandle;

my $TYPE = "GlimmerRBSParser"; 
my $NAME = "Gl-RBS";
my $DIFF=1;

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Whole data from given file is read and stored as Element-objects to a new
# created DataHandle. Start is the first base after start codon, end 
# position is before terminator. This is changed to first position of the
# terminator by $DIFF nuc.
# score is shift (shifting startposition upstream to 3' end)
# comment is position of rbs
#@param filename of glimmerRBS-output
#@return current DataHandle 
#------------------------
sub parse{
    my ($self, $file) = @_;
    my $data_handle = DataHandle->new($TYPE);
    my @array;
    my $diff;
    open(g_input, $file) 
       || die "cannot open glimmer input file ".$file."\n";
    while (<g_input>){last if(/ID/i);}
    while (<g_input>){
	@array = split;
	my $com = $self->complement($array[1], $array[2]);
	if ($com=~/R/i){
	    $diff=-$DIFF;
	}
	else{
	    $diff=$DIFF;
	}
	$array[6]='-' if(!$array[6]);
  	$data_handle->add_element($array[0], $array[1], ($array[2]+$diff), $com, 
				  $array[6], $array[4]);
    }
    close(g_input);
    return $data_handle;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Calulates the complement of given start - end position. 
# (for deprecated routin with orf nr see CriticaParser::complement_no())
# @param $start
# @param $end
# @return complement (format: F2)
#---------------------------
sub complement{
    my ($self, $start, $end) = @_;
    return 'F' if ($start<$end);
    return 'R';
}

sub name{
    return $NAME;
}

sub type{
    return $TYPE;
}
return 1;
