#!/usr/bin/perl

package Nudge;
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Superclass of starter objects for prediction tools.
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*

my $BASH_CALL = "#!/bin/bash\n\n";

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @param FASTA
# @param OUTPUR_DIR 
# @param PREFIX
#---------------------
sub new{
    my $self = { };
    bless $self, shift;
    $self->{FASTA} = shift;      #should be absolute pathname
    $self->{OUTPUT_DIR} = shift;
    $self->{PREFIX} = shift;
    #default, should be overwritten in inheriting classes
    $self->{CALL_SCRIPT} = $ENV{HOME}."/bin/call.sh"; 
    $self->{RESULT_OUTPUT} = $self->{OUTPUT_DIR}."/".$self->{PREFIX}
                             .".def_out";
    return $self; 
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @return path of output file
#-----------------------
sub get_resultoutput{
    my $self = shift;
    return $self->{RESULT_OUTPUT};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# getting/setting output directory
# @return path of output directory
#-----------------------
sub output_dir{
    my ($self, $tmp) = @_;
    $self->{OUTPUT_DIR} = $tmp if($tmp);
    return $self->{OUTPUT_DIR};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# prints bash call in call_script.
#-----------------------
sub print_bash{
    my $self = shift;
    open(call, "> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
         || return 0;
    print call $BASH_CALL;
    close(call);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#@return name of the call script 
#-----------------------
sub call_script_name{
    my $self = shift;
    return $self->{CALL_SCRIPT};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts call script as qsub process.
#-----------------------
sub start_script{
    my $self = shift;
    system "chmod a+rx ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}
           .".sh";     #make script executable
    system "qsub -q dque ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}
           .".sh -o ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT} 
           .".o -e ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".e -V";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @return path of FASTA-file
#-----------------------
sub fasta{
    my ($self, $tmp) = @_;
    $self->{FASTA} = $tmp if ($tmp);
    return $self->{FASTA};
}
return 1;
