#!/usr/bin/perl

package OrpheusNudge;
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
# author mt
# Starter for Orpheus2.
# offers routines to start all the subprograms of orpheus2 manually or to
# let them start by a batch script. note that program 'starter' in 
# MODE_NUC is only for calculation of the nucleotid usage, which should 
# be done first caus it's needed for the calls of the other subprograms.
# it's also possible to call the script-writing routines manually. therefor
# routine print_bash() from superclass Nudge has to be called first and as
# mentioned above, program starter in MODE_NUC. 
#=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*=*
@ISA=(Nudge);
$|=1;

my $MODE_NUC=0;           #for calculation of nucleotid usages
my $MODE_WEIGHTS=1;       #for calculation of weights
my $MODE_WCU=2;           #first orpheus2
my $MODE_SURE=3;          #second orpheus2
my $MODE_RCU=4;           #third orpheus2

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @param FASTA
# @param OUTPUT_DIR       
# @param PREFIX
# @param FASTA_DIR        # NOTE: /xyz/xyz/ all other dirs /xyz/xyz
# @param FASTA_NAME       
#---------------------
sub new{
    my $self = shift;
    $self = $self->SUPER::new(shift, shift, shift);
    $self->{FASTA_DIR} = shift;
    $self->{FASTA_NAME} = shift;
    $time = time; #to make script name unique
    $self->{CALL_SCRIPT} = "sh_o".$time;
    $self->{DPS_OUT} = $self->{OUTPUT_DIR}."/orph_".$self->{PREFIX}.".dps";
    $self->{CODON_USAGE} = $self->{OUTPUT_DIR}."/orph_".$self->{PREFIX}
                           .".codon_usage";
    $self->{NUC_USAGE} = $self->{OUTPUT_DIR}."/orph_".$self->{PREFIX}
                         .".nuc_usage";
    $self->{WEIGHTS} = $self->{OUTPUT_DIR}."/orph_".$self->{PREFIX}
                        .".weights";
                        #orpheus writes output to fasta dir
    $self->{RESULT_OUTPUT} = $self->{OUTPUT_DIR}."/".$self->{FASTA_NAME}
                             .".orfnuc";
    return $self;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# script for batch processes of all sub 'programs' is written and started
# with qsub
#-----------------------
sub start_qsub_all{
    my $self = shift;
    $self->start_starter(OrpheusNudge->nuc());   #calculate nucleotid usage
    $self->print_bash();
    $self->script_dps();
    $self->script_orpheus2(OrpheusNudge->wcu());
    $self->script_orpheus2(OrpheusNudge->sure());
    $self->script_starter(OrpheusNudge->weights());
    $self->script_orpheus2(OrpheusNudge->rcu());
    $self->start_script();
}


#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# To move results to outputdir, caus orpheus writes results to fasta dir. 
# BE AWARE: all FASTA_NAME.orf* files are moved to current output directory
#-----------------------
sub move_result(){
    my $self = shift;
    system "mv ".$self->{FASTA_DIR}.$self->{FASTA_NAME}.".orf* "
	.$self->{OUTPUT_DIR}."/";
    $self->{RESULT_OUTPUT}=$self->{OUTPUT_DIR}."/".$self->{FASTA_NAME}
                           .".orfnuc";
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for dps batch process.
# NOTE for manual calls: print_bash() should be called first.  
#-----------------------
sub script_dps{
   my $self = shift;

   open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
         || return 0;           
   print call $self->{PARAMETER}{dps_home}."/dps ".$self->{FASTA}." "
             .$self->{PARAMETER}{dps_nr}." "
                    .$self->{PARAMETER}{dps_BLOSUM}
                    .$self->{PARAMETER}{dps_f}
                    .$self->{PARAMETER}{dps_c}.$self->{PARAMETER}{dps_a}
                    .$self->{PARAMETER}{dps_d}.$self->{PARAMETER}{dps_i}
                    .$self->{PARAMETER}{dps_w}." > ".$self->{DPS_OUT}."\n";
   close(call);
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for starter batch processes
# here no mode is necessary caus MODE_NUC is only to calculate nucleotid 
# usage, which is needed for the other calls.
# NOTE for manual calls: print_bash() should be called first.  
#-----------------------
sub script_starter{
    my $self = shift;
    open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
          || return 0;           
    if (!$self->{NUC}){
	if ($self->nuc_usage() == 0){close(call);return 0;}}
    my @key = keys %{$self->{NUC}};
    print call $self->{PARAMETER}{orph_home}."/starter -m ".$key[0]
	.$self->{NUC}{$key[0]}." ".$key[1].$self->{NUC}{$key[1]}." "
	    .$key[2].$self->{NUC}{$key[2]}." ".$key[3]
            .$self->{NUC}{$key[3]}." "
		.$self->{FASTA_DIR}.$self->{FASTA_NAME}.".orfaln > "
                .$self->{WEIGHTS}."\n";
    close(call);
} 

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# writes script for orpheus2 batch processes
# NOTE for manual calls: print_bash() should be called first.  
# @param mode
#-----------------------
sub script_orpheus2{
    my $self=shift;
    my $mode = shift;
    open(call, ">> ".$self->{OUTPUT_DIR}."/".$self->{CALL_SCRIPT}.".sh") 
         || return 0;           
    if ($mode == $MODE_WCU){
	if (!$self->{NUC}){
	    if ($self->nuc_usage() == 0){close(call); return 0;}}
	my @key = keys %{$self->{NUC}};
	print call "perl ".$self->{PARAMETER}{orph_home}
                   ."/orpheus2.pl -genome "
	           .$self->{FASTA}." ".$key[0]." ".$self->{NUC}{$key[0]}
                   ." ".$key[1]." ".$self->{NUC}{$key[1]}." "
                   .$key[2]." ".$self->{NUC}{$key[2]}." "
		   .$key[3]." ".$self->{NUC}{$key[3]}." -wcu "
		   .$self->{CODON_USAGE}." "
	           .$self->{PARAMETER}{orph_minlength}
#	           .$self->{PARAMETER}{orph_minscore}
#	           .$self->{PARAMETER}{orph_verbose}
#	           .$self->{PARAMETER}{orph_keep}
#	           .$self->{PARAMETER}{orph_percentid}
#	           .$self->{PARAMETER}{orph_trusted}
#	           .$self->{PARAMETER}{orph_maxoverlap}
#	           .$self->{PARAMETER}{orph_excl}
		       .$self->{DPS_OUT}."\n";
    }
    elsif ($mode == $MODE_SURE){
	if (!$self->{NUC}){
	    if ($self->nuc_usage() == 0){close(call); return 0;}}
	my @key = keys %{$self->{NUC}};
	print call "perl ".$self->{PARAMETER}{orph_home}
                    ."/orpheus2.pl -genome "
	            .$self->{FASTA}." ".$key[0]." ".$self->{NUC}{$key[0]}
                    ." ".$key[1]." ".$self->{NUC}{$key[1]}." "
                    .$key[2]." ".$self->{NUC}{$key[2]}." "
		    .$key[3]." ".$self->{NUC}{$key[3]}." -rcu "
		    .$self->{CODON_USAGE}." -sure "
	           .$self->{PARAMETER}{orph_minlength}
#	           .$self->{PARAMETER}{orph_minscore}
#	           .$self->{PARAMETER}{orph_verbose}
#	           .$self->{PARAMETER}{orph_keep}
#	           .$self->{PARAMETER}{orph_percentid}
#	           .$self->{PARAMETER}{orph_trusted}
#	           .$self->{PARAMETER}{orph_maxoverlap}
#	           .$self->{PARAMETER}{orph_excl}
			.$self->{DPS_OUT}."\n";
    }
    elsif($mode == $MODE_RCU){
	if (! $self->{NUC}){
	    if ($self->nuc_usage() == 0){close(call); return 0;}}
	my @key = keys %{$self->{NUC}};
	print call "perl ".$self->{PARAMETER}{orph_home}
                   ."/orpheus2.pl -genome "
	           .$self->{FASTA}." ".$key[0]." "
                   .$self->{NUC}{$key[0]}." ".$key[1]." "
		   .$self->{NUC}{$key[1]}." ".$key[2]." "
                   .$self->{NUC}{$key[2]}." "
		   .$key[3]." ".$self->{NUC}{$key[3]}." -rcu "
		   .$self->{CODON_USAGE}." -w ".$self->{WEIGHTS}." "
	           .$self->{PARAMETER}{orph_minlength}
#	           .$self->{PARAMETER}{orph_minscore}
#	           .$self->{PARAMETER}{orph_verbose}
#	           .$self->{PARAMETER}{orph_keep}
#	           .$self->{PARAMETER}{orph_percentid}
#	           .$self->{PARAMETER}{orph_trusted}
#	           .$self->{PARAMETER}{orph_maxoverlap}
#	           .$self->{PARAMETER}{orph_excl}
		   .$self->{DPS_OUT}."\n";
    }
    else{
	close(call);
	return 0;
    }
    close(call);
    return 1;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @return hash with nuleotides as key and their frequency as value
#         (example: -a, 0.31)
#-----------------------
sub nuc_usage{
    my $self = shift;
    if (!$self->{NUC}){
	open (nuc, $self->{NUC_USAGE}) || return 0;
	%{$self->{NUC}} = split " ", <nuc>;    
    }
    return $self->{NUC};
}
 
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts dps as non batch process.
#-----------------------
sub start_dps{
    my $self = shift;
    system $self->{PARAMETER}{dps_home}."/dps ".$self->{FASTA}." "
	.$self->{PARAMETER}{dps_nr}." "
	.$self->{PARAMETER}{dps_BLOSUM}.$self->{PARAMETER}{dps_f}
        .$self->{PARAMETER}{dps_c}.$self->{PARAMETER}{dps_a}
        .$self->{PARAMETER}{dps_d}.$self->{PARAMETER}{dps_i}
        .$self->{PARAMETER}{dps_w}." > ".$self->{DPS_OUT};
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts orpheus2 in given mode as non batch process.
# @param $mode
#-----------------------
sub start_orpheus2{
   my $self = shift;
   my $mode = shift;
   if ($mode == $MODE_WCU){
       if (!$self->{NUC}){
	   if ($self->nuc_usage() == 0){ return 0;}}
       my @key = keys %{$self->{NUC}};
       system "perl ".$self->{PARAMETER}{orph_home}
             ."/orpheus2.pl -genome ".$self->{FASTA}
             ." ".$key[0]." ".$self->{NUC}{$key[0]}." ".$key[1]." "
	     .$self->{NUC}{$key[1]}." ".$key[2]." "
             .$self->{NUC}{$key[2]}." "
	     .$key[3]." ".$self->{NUC}{$key[3]}." -wcu "
		 .$self->{CODON_USAGE}
#       .$self->{PARAMETER}{orph_minlength}
#       .$self->{PARAMETER}{orph_minscore}
#       .$self->{PARAMETER}{orph_verbose}
#       .$self->{PARAMETER}{orph_keep}
#       .$self->{PARAMETER}{orph_percentid}
#       .$self->{PARAMETER}{orph_trusted}
#       .$self->{PARAMETER}{orph_maxoverlap}
#       .$self->{PARAMETER}{orph_excl}
             ." ".$self->{DPS_OUT}."\n";
   }
   elsif ($mode == $MODE_SURE){
       if (!$self->{NUC}){
	   if ($self->nuc_usage() == 0){return 0;}}
       my @key = keys %{$self->{NUC}};
       system "perl ".$self->{PARAMETER}{orph_home}
              ."/orpheus2.pl -genome ".$self->{FASTA}
              ." ".$key[0]." ".$self->{NUC}{$key[0]}." ".$key[1]." "
	      .$self->{NUC}{$key[1]}." ".$key[2]." "
              .$self->{NUC}{$key[2]}." "
	      .$key[3]." ".$self->{NUC}{$key[3]}
       ." -rcu ".$self->{CODON_USAGE}
#       .$self->{PARAMETER}{orph_minlength}
#       .$self->{PARAMETER}{orph_minscore}
#       .$self->{PARAMETER}{orph_verbose}
#       .$self->{PARAMETER}{orph_keep}
#       .$self->{PARAMETER}{orph_trusted}
#       .$self->{PARAMETER}{orph_maxoverlap}
#       .$self->{PARAMETER}{orph_excl}
              ." -sure ".$self->{DPS_OUT}."\n";
   }
   elsif($mode == $MODE_RCU){
       if (! $self->{NUC}){
	   if ($self->nuc_usage() == 0){return 0;}}
       my @key = keys %{$self->{NUC}};
       system "perl ".$self->{PARAMETER}{orph_home}
              ."/orpheus2.pl -genome ".$self->{FASTA}
              ." ".$key[0]." ".$self->{NUC}{$key[0]}." ".$key[1]." "
	      .$self->{NUC}{$key[1]}." ".$key[2]." "
              .$self->{NUC}{$key[2]}." "
	      .$key[3]." ".$self->{NUC}{$key[3]}." -rcu "
              .$self->{CODON_USAGE}
#       .$self->{PARAMETER}{orph_minlength}
#       .$self->{PARAMETER}{orph_minscore}
#       .$self->{PARAMETER}{orph_verbose}
#       .$self->{PARAMETER}{orph_keep}
#       .$self->{PARAMETER}{orph_percentid}
#       .$self->{PARAMETER}{orph_trusted}
#       .$self->{PARAMETER}{orph_maxoverlap}
#       .$self->{PARAMETER}{orph_excl}
              ." -w ".$self->{WEIGHTS}." ".$self->{DPS_OUT}."\n";
   }
   else{
       return 0;
   }
   return 1;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# starts starter in given mode as non batch process. 
#@param mode
#-----------------------
sub start_starter{
   my $self = shift;
   my $mode = shift;
   if ($mode == $MODE_NUC){
       system $self->{PARAMETER}{orph_home}."/starter -q ".$self->{FASTA}
       ." > ".$self->{NUC_USAGE};
   }
   elsif ($mode == $MODE_WEIGHTS){
       if (!$self->{NUC}){
	   if ($self->nuc_usage() == 0){return 0;}}
       my @key = keys %{$self->{NUC}};
       system $self->{PARAMETER}{orph_home}."/starter -m ".$key[0]
	   .$self->{NUC}{$key[0]}." ".$key[1].$self->{NUC}{$key[1]}." "
	   .$key[2].$self->{NUC}{$key[2]}." ".$key[3]
           .$self->{NUC}{$key[3]}." "
	   .$self->{FASTA_DIR}.$self->{FASTA_NAME}.".orfaln > "
           .$self->{WEIGHTS};
   }
   else{
       return 0;
   }
   return 1;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# evaluation of given properties.
# @see orpheus documentation and ini file
# @param properties as hash
#----------------------
sub set_parameter{
    my $self = shift;
    $self->{PARAMETER} = shift;

    if (! $self->{PARAMETER}{dps_home}){         #dps_home
	return 0;}

    if (! $self->{PARAMETER}{dps_nr}){         #dps_nr
	return 0;}
    
    if (! $self->{PARAMETER}{dps_BLOSUM}){     #dps_BLOSUM
	return 0;}
    
    if ( $self->{PARAMETER}{dps_f}){           #dps_f 
	$self->{PARAMETER}{dps_f} = " -f ".$self->{PARAMETER}{dps_f}." ";}
    else{ return 0;}
    
    if ( $self->{PARAMETER}{dps_c}){           #dps_c 
	$self->{PARAMETER}{dps_c} = " -c ".$self->{PARAMETER}{dps_c}." ";}
    else{ return 0; }
    
    if ( $self->{PARAMETER}{dps_a}){           #dps_a 
	$self->{PARAMETER}{dps_a} = " -a ".$self->{PARAMETER}{dps_a}." ";}

    if ( $self->{PARAMETER}{dps_d}){           #dps_d
	$self->{PARAMETER}{dps_d} = " -d ".$self->{PARAMETER}{dps_d}." ";}
    
    if ( $self->{PARAMETER}{dps_i}){           #dps_i 
	$self->{PARAMETER}{dps_i} = " -i ".$self->{PARAMETER}{dps_i}." ";}    
    
    if ( $self->{PARAMETER}{dps_w}){           #dps_w 
	$self->{PARAMETER}{dps_w} = " -w ".$self->{PARAMETER}{dps_w}." ";}    
    
    if (! $self->{PARAMETER}{orph_home}){      #orph_home
	return 0;}

    if ( $self->{PARAMETER}{orph_verbose}){    #orph_verbose
	$self->{PARAMETER}{orph_verbose} = " -verbose ";}
 
    if ( $self->{PARAMETER}{orph_minlength}){  #orph_minlength
	$self->{PARAMETER}{orph_minlength} = " -minlength "
	    .$self->{PARAMETER}{orph_minlength}." ";}

    if ( $self->{PARAMETER}{orph_minscore}){   #orph_minscore
	$self->{PARAMETER}{orph_minscore} = " -minscore "
	    .$self->{PARAMETER}{orph_minscore}." ";}
    
    if ( $self->{PARAMETER}{orph_keep}){       #orph_keep
	$self->{PARAMETER}{orph_keep} = " -keep ".
                                       $self->{PARAMETER}{orph_keep}." ";}
    
    if ( $self->{PARAMETER}{orph_percentid}){  #orph_percentid
	$self->{PARAMETER}{orph_percentid}= " -percentid "
	    .$self->{PARAMETER}{orph_percentid}." ";}

    if ( $self->{PARAMETER}{orph_trusted}){    #orph_trusted
	$self->{PARAMETER}{orph_trusted} = " -trusted "
	    .$self->{PARAMETER}{orph_trusted}." ";}

    if ( $self->{PARAMETER}{orph_maxoverlap}){ #orph_maxoverlap
	$self->{PARAMETER}{orph_maxoverlap} = " -maxoverlap "
	    .$self->{PARAMETER}{orph_maxoverlap}." ";}

    if ( $self->{PARAMETER}{orph_excl}){        #orph_excl
	$self->{PARAMETER}{orph_excl} = " -excl ".
                                        $self->{PARAMETER}{orph_excl}." ";}
    return 1;
}

sub nuc{
    return $MODE_NUC;
}

sub weights{
    return $MODE_WEIGHTS;
}

sub wcu{
    return $MODE_WCU;
}

sub sure{
    return $MODE_SURE;
}

sub rcu{
    return $MODE_RCU;
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @TODO
#-----------------------
sub script_tRNAScan{

}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# @TODO
#-----------------------
sub start_tRNAScan{

}

